@@ -646,6 +646,12 @@ int git_stash_drop(
 	if (max == 1) {
 		error = git_reference_delete(stash);
 		stash = NULL;
+	} else if (index == 0) {
+		const git_reflog_entry *entry;
+
+		entry = git_reflog_entry_byindex(reflog, 0);
+
+		error = git_reference_set_target(stash, &entry->oid_cur);
 	}
 
 cleanup:
